/*************************************************************************************
	NOTEMIDITONE - plays a test tone to an audio device or file

	(C) 2001 by Michael Minn - michael@michaelminn.com

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

*************************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <math.h>
#include <string.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <linux/soundcard.h>
#include "wave_file_header.h"

int set_audio_device_44khz_stereo(int device_file)
{
	int audio_format = AFMT_S16_LE; /* 16-bit signed little-endian (Intel standard) */
	if (ioctl(device_file, SNDCTL_DSP_SETFMT, &audio_format) < 0)
		perror("Failure setting 16-bit format - no problem if writing to a file");

	int channels = 2;
	if (ioctl(device_file, SNDCTL_DSP_CHANNELS, &channels) < 0)
		perror("Failure setting stereo format - no problem if writing to a file");

	int sampling_rate = 44100;
	if (ioctl(device_file, SNDCTL_DSP_SPEED, &sampling_rate) < 0)
		perror("Failure setting 44.1 Khz format - no problem if writing to a file");

	return 0;
}

int main(int argc, char **argv)
{
	if ((argc < 4) || (argc > 5) || ((argc == 5) && (strcmp(argv[1], "-q") != 0)))
	{
		printf("SYNTAX: notemiditone [-q] <audio device name> <frequency> <# of samples>\n");
		printf("   plays a test tone in 44khz PCM stereo to the specified audio device.\n");
		printf("   -q option gives square wave instead of sine wave.\n");
		return -1;
	}

	char *file_name = argv[1];
	int frequency = atoi(argv[2]);
	int samples = atoi(argv[3]);
	int is_sine_wave = 1;

	if (argc == 5)
	{
		file_name = argv[2];
		frequency = atoi(argv[3]);
		samples = atoi(argv[4]);
		is_sine_wave = 0;
	}

	int file = open(file_name, O_WRONLY | O_TRUNC | O_CREAT);
	wave_file_header header;

	header.data_chunk_length = (samples * 4) - sizeof(wave_file_header);
	header.header_chunk_length = header.data_chunk_length + 36;

	if (file < 0)
		perror(file_name);

	else if ((frequency <= 0) || (frequency > 22050))
		printf("Frequency must be 1 - 22050 Hz\n");

	else if (samples <= 8)
		printf("Number of samples must be >= 8\n");

	else if (write(file, &header, sizeof(header)) < sizeof(header))
		perror("Failure writing header");

	else if (set_audio_device_44khz_stereo(file) < 0)
		perror("Failure device format");

	else for (int sample = 0; sample < samples; ++sample)
	{
		short data[2];
		if (is_sine_wave)
			data[0] = (int)(32000.0 * sin(2.0 * M_PI * (double)sample * (double)frequency / 44100.0));
		else if ((sample % frequency) < (frequency / 2))
			data[0] = 32000;
		else
			data[0] = -32000;

		data[1] = data[0];

		if (write(file, data, 4) < 4)
		{
			perror("Failure writing data\n");
			return -1;
		}
	}

	return 0;
}
